/*
 * Read Flash ROM on JavaStation-E
 * This is basically a dd iseek=xxx;
 * Except that GNU dd has no iseek parameter.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#if 0
#include <sys/types.h>
#include <sys/mman.h>
#endif
#include <errno.h>
#include <fcntl.h>

#define FLASHPHYS  0x20400000
#define MEG        0x00100000
#define BUFSZ      8192

void Usage(void);

char *dev = "/dev/jsflash";

void
main(int argc, char **argv)
{
	int fd, ofd;
	void *mp;
	char *out = 0;
	char *es;
	unsigned int cnt;
	int rc;

	if (argc == 2) {
		out = argv[1];
	} else {
		Usage();
	}

	if ((fd = open(dev, O_RDONLY)) == -1) {
		if ((es = strerror(errno)) == NULL) es = "???";
		fprintf(stderr, "flashrd: open(%s): %s\n", dev, es);
		exit(1);
	}

	if ((ofd = open(out, O_WRONLY|O_TRUNC|O_CREAT, 0644)) == -1) {
		if ((es = strerror(errno)) == NULL) es = "???";
		fprintf(stderr, "flashrd: open(%s): %s\n", out, es);
		exit(1);
	}

#if 0
	mp = mmap(NULL, 8192, PROT_READ, 0, fd, FLASHPHYS);
	if (mp == (void *)(-1)) {
		perror("flashrd: mmap");
		exit(1);
	}
	.....
	munmap(mp, 8192);
#endif

	if ((mp = malloc(BUFSZ)) == NULL) {
		fprintf(stderr, "flashrd: no core\n");
		exit(1);
	}

	lseek(fd, FLASHPHYS, SEEK_SET);
	for (cnt = 0; cnt < 4*MEG; cnt += BUFSZ) {
		if ((rc = read(fd, mp, BUFSZ)) < 0) {
			if ((es = strerror(errno)) == NULL) es = "???";
			fprintf(stderr, "flashrd: read: %s\n", es);
			exit(1);
		}
		if (rc < BUFSZ) {
			fprintf(stderr, "flashrd: read short %d\n", rc);
			exit(1);
		}

		if ((rc = write(ofd, mp, BUFSZ)) < 0) {
			if ((es = strerror(errno)) == NULL) es = "???";
			fprintf(stderr, "flashrd: write: %s\n", es);
			exit(1);
		}
		if (rc < BUFSZ) {
			fprintf(stderr, "flashrd: write short %d\n", rc);
			exit(1);
		}
		printf("%u\r", cnt);
	}
	printf("\n");

	free(mp);
	close(fd);
	close(ofd);
	exit(0);
}

void Usage() {
	fprintf(stderr, "Usage: flashrd outfile\n");
	exit(1);
}
